# worklight-access-token-validator

This node module purpose is to allow for an offline validation of Worklight server generated OAuth tokens.

## Usage

Usage of this module requires that you have access to the certificate used in the server for signing the tokens.
To extract the certificate from a JKS keystore you can use Java's keytool in the following manner:

```bash
keytool -exportcert -keystore $KEYSTORE_FILE -alias $CERTIFICATE_ALIAS -rfc > $OUTPUT_FILE.pem
```

You then have to pass the the content of the generated (PEM formatted) certificate file as input to the node module which will allow you to validate tokens created with the same certificate.

Please note that expiration of the token is checked against the local computer time so make sure your clock is synced (preferably using an NTP server).

### Sample usage:
# After installing the package with 'npm install *'

```javascript
// Load the certificate from a PEM encoded file
var cert = require("fs").readFileSync('cert.pem');

// The scope to mandate (can be null, in which case the token is only checked for a suitable signature and token expiration)
var scope = "WorlightSecurityTest";

// Create a reusable validator
var worklightValidator = require("worklight-access-token-validator")(cert, scope), fs = require("fs");


worklightValidator.validate(token, function(error, payload) {
	if (error) {
		// Token is invalid, send appropriate response to user
		response.writeHead(error.httpStatus,{"WWW-Authenticate":error.wwwAuthenticateHeader});
	} else {
		// Token is valid, proceed with request
	}
});
```

### Error object

The error object in the callback method contains the following fields:
+ err - contains the error code, which can be one of:
	+ 'invalid_token'
	+ 'missing_token'
	+ 'insufficient_scope'
+ errMsg - contains the human readable description of the error reason.
+ httpStatus - the http status to use when responding to the access token sender
+ wwwAuthenticateHeader - the content of the 'WWW-Authenticate' header that should be responded to the access token sender

